"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Query = void 0;
const useAxios_1 = require("./useAxios");
const vue_1 = require("vue");
const vue_query_1 = require("@tanstack/vue-query");
const PaginatedData_1 = require("./PaginatedData");
class Query {
    constructor(queryBuilder) {
        this.queryBuilder = queryBuilder;
    }
    getModel({ select, dependsOn = [], }) {
        const { axios } = (0, useAxios_1.useAxios)();
        return (0, vue_query_1.useQuery)({
            queryKey: this.queryBuilder.keys,
            queryFn: () => axios
                .get(this.queryBuilder.path.value, {
                params: this.queryBuilder.params.value,
            })
                .then((data) => data.data),
            select: select,
            enabled: (0, vue_1.computed)(() => dependsOn === null || dependsOn === void 0 ? void 0 : dependsOn.every((ref) => ref.value !== null && ref.value !== undefined)),
        });
    }
    getPaginatedModels({ select, dependsOn = [], }) {
        const { axios } = (0, useAxios_1.useAxios)();
        const total = (0, vue_1.ref)(undefined);
        const query = (0, vue_query_1.useInfiniteQuery)({
            queryKey: this.queryBuilder.keys,
            queryFn: ({ pageParam = 1 }) => {
                return axios
                    .get(this.queryBuilder.path.value, {
                    params: Object.assign({ "page[number]": pageParam }, this.queryBuilder.params.value),
                })
                    .then((resp) => {
                    total.value = resp.data.total;
                    return resp.data;
                });
            },
            initialPageParam: 1,
            getNextPageParam: (lastPage, _pages) => lastPage.next_page_url ? lastPage.current_page + 1 : undefined,
            select: ({ pageParams, pages }) => {
                var _a;
                total.value = (_a = pages[0]) === null || _a === void 0 ? void 0 : _a.total;
                const transformedPages = pages.map((data) => {
                    return new PaginatedData_1.PaginatedData(data, select);
                });
                return { pages: transformedPages, pageParams };
            },
            enabled: (0, vue_1.computed)(() => dependsOn === null || dependsOn === void 0 ? void 0 : dependsOn.every((ref) => ref.value !== null && ref.value !== undefined)),
        });
        const data = (0, vue_1.computed)(() => { var _a; return (_a = query.data.value) === null || _a === void 0 ? void 0 : _a.pages.flat(); });
        const models = (0, vue_1.computed)(() => { var _a; return (_a = data.value) === null || _a === void 0 ? void 0 : _a.map((item) => item.data).flat(); });
        return Object.assign(Object.assign({}, query), { data,
            models,
            total });
    }
    getAllModels({ select, dependsOn = [], }) {
        const isLoadingLocal = (0, vue_1.ref)(true);
        const results = (0, vue_1.ref)();
        const { data, hasNextPage, fetchNextPage, isRefetching, total, refetch } = this.getPaginatedModels({ select, dependsOn });
        (0, vue_1.watch)([hasNextPage, (0, vue_1.computed)(() => { var _a; return (_a = data.value) === null || _a === void 0 ? void 0 : _a.length; }), isRefetching], () => {
            var _a;
            if ((hasNextPage === null || hasNextPage === void 0 ? void 0 : hasNextPage.value) && !(isRefetching === null || isRefetching === void 0 ? void 0 : isRefetching.value)) {
                fetchNextPage();
            }
            if (!(hasNextPage === null || hasNextPage === void 0 ? void 0 : hasNextPage.value) && !(isRefetching === null || isRefetching === void 0 ? void 0 : isRefetching.value)) {
                isLoadingLocal.value = false;
                results.value = (_a = data.value) === null || _a === void 0 ? void 0 : _a.flatMap((item) => item.data);
            }
        });
        return {
            data: (0, vue_1.computed)(() => { var _a; return (_a = results.value) !== null && _a !== void 0 ? _a : []; }),
            isLoading: isLoadingLocal,
            isRefetching,
            total,
            refetch,
        };
    }
}
exports.Query = Query;
